#include "Add_builder_impl.h"

#include "Log.h"

#include "Defs.h"
#include "Create.h"
#include "Atom.h"
#include "Atom_kit.h"
#include "Molecule.h"
#include "Vector_3D_impl.h"
#include "View_atom_type.h"
#include "View_atom_type_manager.h"
#include "HSV.h"
#include "Length.h"
#include "Add_model_dialog.h"

namespace MM
{

Add_builder_impl::
Add_builder_impl (Prototype const &)
:
    model_(nil<Model>()),
    translation_increment_  (0.),
    rotation_increment_     (0.),
    overlap_allowed_        (true)
{
}

Add_builder_impl::
Add_builder_impl (Model & model)
:
    model_(model),
    new_model_ (prototype <Model> (). clone()),
    translation_increment_  (1),
    rotation_increment_     (6 * __Pi__ / 180.), //6      //fix __Pi__
    overlap_allowed_        (true)
{
    /*Atom * atom_prototype = model.atom_prototype().clone();
    atom_prototype->set_element ("Dummy");
    //atom_prototype->kit().visual().show();//fix
    new_model_().set_atom_prototype (atom_prototype);//*/
}

Add_builder * Add_builder_impl::
clone (Model & model) const
{
    return new Add_builder_impl (model);
}

void Add_builder_impl::
load (const char * filename)
{
    Atom * atom_prototype = model_.atom_prototype().clone();
    atom_prototype->set_element ("Dummy");
    //atom_prototype->kit().visual().show();//fix
    new_model_().set_atom_prototype (atom_prototype);

    new_model_().load (filename);

    double kS = 0.3, kV = 0.6, kR = 0.5;

    for (int n=0;  n<new_model_().atom_count();  ++n)
    {
        Atom &current_atom = new_model_().atom (n);

        View_atom_type type = View_atom_type_manager::instance().
            type (current_atom.element().c_str());     //fix speed

        double R = type.r(),   G = type.g(),   B = type.b();
        double H,S,V;
        RGB_to_HSV (R,G,B, H,S,V);
        S *= kS;
        V *= kV;
        HSV_to_RGB (H,S,V, R,G,B);
        type.set_color ((int)R,(int)G,(int)B);
        double radius = type.radius();
        type.set_radius (radius * kR);

        current_atom.kit().visual().update_view_type (type);
    }
    user_interaction ();
}

void Add_builder_impl::
reject ()
{
    new_model_().clear (); //gix clear to erase ?
}

void Add_builder_impl::
accept ()
{
    for (int n=0;  n<new_model_().atom_count();  ++n)
    {
        Atom &current_atom = new_model_().atom (n);

        View_atom_type type = View_atom_type_manager::instance().
            type (current_atom.element().c_str() );     //fix speed

        current_atom.kit().visual().update_view_type (type);
    }

    if (!overlap_allowed_)
        check_overlap (true);

    model_.merge_model (&new_model_());
}

bool Add_builder_impl::
is_active () const
{
    return new_model_().atom_count() != 0;
}

void Add_builder_impl::
allow_overlap (bool on)
{
    overlap_allowed_ = on;
    
    // show all 
    for (int i=0;  i<new_model().atom_count();  ++i)
        new_model().atom(i).kit().visual().show();

    check_overlap ();
}

void Add_builder_impl::
up ()
{
    model_.move (Vector_3D_impl (0, translation_increment_, 0));
    check_overlap ();
    //repaint
}

void Add_builder_impl::
down ()
{
    model_.move (Vector_3D_impl (0, -translation_increment_, 0));
    check_overlap ();
}

void Add_builder_impl::
right ()
{
    model_.move (Vector_3D_impl (translation_increment_, 0, 0));
    check_overlap ();
}

void Add_builder_impl::
left ()
{
    model_.move (Vector_3D_impl (-translation_increment_, 0, 0));
    check_overlap ();
}

void Add_builder_impl::
out ()
{
    model_.move (Vector_3D_impl (0, 0, translation_increment_));
    check_overlap ();
}

void Add_builder_impl::
in ()
{
    model_.move (Vector_3D_impl (0, 0, -translation_increment_));
    check_overlap ();
}

void Add_builder_impl::
right_X ()
{
    model_.rotate (Vector_3D_impl (1, 0, 0), rotation_increment_);
}

void Add_builder_impl::
left_X ()
{
    model_.rotate (Vector_3D_impl (-1, 0, 0), rotation_increment_);
}

void Add_builder_impl::
right_Y ()
{
    model_.rotate (Vector_3D_impl (0, 1, 0), rotation_increment_);
}

void Add_builder_impl::
left_Y ()
{
    model_.rotate (Vector_3D_impl (0, -1, 0), rotation_increment_);
}

void Add_builder_impl::
right_Z ()
{
    model_.rotate (Vector_3D_impl (0, 0, 1), rotation_increment_);
}

void Add_builder_impl::
left_Z ()
{
    model_.rotate (Vector_3D_impl (0, 0, -1), rotation_increment_);
}

double Add_builder_impl::
translation_increment () const
{
    return translation_increment_;
}

double Add_builder_impl::
rotation_increment () const
{
    return rotation_increment_;
}

void Add_builder_impl::
set_translation_increment (double value)
{
    translation_increment_ = value;
}

void Add_builder_impl::
set_rotation_increment (double value)
{
    rotation_increment_ = value;
}

void Add_builder_impl::
user_interaction ()
{
    singleton <Add_model_dialog>(). execute();
}

void Add_builder_impl::
check_overlap (bool remove)
{
    if (overlap_allowed_)
        return;

    for (int i=0;  i<new_model().atom_count();  ++i)
    {
        Atom & new_atom        = new_model().atom(i);
        
        double new_atom_radius = 
            .5 * (new_atom.element().covalent_radius() 
                + new_atom.element().van_der_Waals_radius ()); 

        new_atom.kit().visual().show();

        for (int j=0;   j<previous_model().atom_count();  ++j)
        {
            Atom & previous_atom        = previous_model().atom(j);

            double previous_atom_radius = 
                .5 * (previous_atom.element().covalent_radius() 
                    + previous_atom.element().van_der_Waals_radius ());

            double distance_2 = Length 
                (new_atom.position(), previous_atom.position()).value_2();

            double radius_sum = new_atom_radius + previous_atom_radius;
            if (distance_2 < radius_sum * radius_sum)
                new_atom.kit().set_overlap (true);
        }
    }

    for (int m=0;  m<new_model().molecule_count();  ++m)
    {
        Molecule & molecule = new_model().molecule(m);

        for (int n=0;  n<molecule.atom_count();  ++n)
        {
            Atom & atom = molecule.atom (n);
            
            if (atom.kit().has_overlap())
            {
                for (int k=0;  k<molecule.atom_count();  ++k)
                {
                    Atom & atom = molecule.atom (k);
                    atom.kit().set_overlap (false);

                    if (remove)
                        atom.set_dead ();
                    else
                        atom.kit().visual().hide();
                }
                continue;
            }
        }
    }

    //fix to kit().molecule
    //                    molecule.remove_atom (atom); 
    //                    new_model().erase (atom);   
    if (remove)
        new_model().remove_deads();

    //fix ENSURE ("No overlap flags.");
}

}//MM





